#!/usr/bin/perl

# Copyright (c) 2009 Jonathan Kamens <jik@kamens.brookline.ma.us>.
#
# May be distributed under the terms of the GNU General Public
# License, Version 3 or any newer version at your discretion.  See
# http://www.gnu.org/licenses/gpl.html for license terms.

# Fetches phone numbers and associated caller IDs from recent calls
# listed on your dashboard at www.vonage.com, and either sends them to
# stdout or to a CSV database.  If the latter, then entries more than
# 180 days old are purged from the database.  Intended to be used with
# vonage-vmail-filter.pl to add names to Vonage voicemail
# notifications.
#
# Specify --username=username, --password=password, and (optionally)
# --database=filename on command line.
#
# Run periodically out of cron or something to keep your database
# up-to-date.
#
# $Id: vonage-cid.pl,v 1.3 2009/12/30 21:48:56 jik Exp $

$expire = 180 * 24 * 60 * 60;

use Getopt::Long;
use HTML::Entities;
use WWW::Mechanize;

die if (! GetOptions("username=s" => \$user,
		     "password|pw=s" => \$pw,
		     "database|db=s" => \$db_file));
die "Must specify username\n" if (! $user);
die "Must specify password\n" if (! $pw);

if ($db_file) {
    open(DB, "<", $db_file) or die;
    <DB>; # header line
    while (<DB>) {
	chomp;
	my($number, $cid, $age) = split(/,/);
	next if (time() - $age > $expire);
	$cid{$number} = $cid;
	$age{$number} = $age;
    }
    close(DB) or die;
}

my $mech = WWW::Mechanize->new();

$mech->get("http://www.vonage.com/?login");

#print $mech->content();
#exit;

$mech->form_name("logonForm") or die;
$mech->set_fields("username" => $user,
		  "password" => $pw);
$mech->click() or die;

$mech->get("https://secure.vonage.com/webaccount/dashboard/index.htm");

#print $mech->content();
#exit;

$_ = $mech->content();

while (/<CENTER>([^<>]+)<br><br><b>([^<>]+)<\/b>/g) {
    my($number, $cid) = ($1,$2);
    next if ($did{$number}++);
    $cid =~ s/\\\&/\&/g;
    $cid{$number} = &decode_entities($cid);
    $age{$number} = time();
}

if ($db_file) {
    open(STDOUT, ">", "$db_file.new") or die;
}

print "number,cid,age\n";

foreach my $number (sort keys %cid) {
    print $number, ",", $cid{$number}, ",", $age{$number}, "\n";
}

close(STDOUT) or die;

if ($db_file) {
    rename("$db_file.new", $db_file) or die;
}

# $Log: vonage-cid.pl,v $
# Revision 1.3  2009/12/30 21:48:56  jik
# Fix option parsing so that username and password command-line options
# are handled correctly.
# Add change log to bottom of script.
#
